#include <NewTone.h>
#include <NewPing.h>

// VARIABLE DECLARATIONS 
const unsigned int SONAR_NUM = 4; // Number or sensors.
const unsigned int MAX_DISTANCE = 200; // Max distance in cm. 
const unsigned int MAX_ALARM_DISTANCE = 40; //Max distance to sound alarm in cm
const unsigned int MIN_ALARM_DISTANCE = 10; //Min distance to sound alarm in cm
const unsigned int MAX_DELAY = 600;// Max delay to sound alarm in milliseconds
const unsigned int MIN_DELAY = 75; // Min delay to sound alarm in milliseconds
const unsigned int SPEAKER_1 = 13; // Speaker pin
const unsigned int SPEAKER_FREQUENCY = 800; // Speaker frequency in hertz
const float slope = (MAX_DELAY - MIN_DELAY) / (MAX_ALARM_DISTANCE - MIN_ALARM_DISTANCE); //Slope of linear function to calculate alarm duration vs distance
const float b = MAX_DELAY - slope * MAX_ALARM_DISTANCE; //y intercept of linear function to calculate alarm duration vs distance

NewPing sonar[SONAR_NUM] = { // Sensor object array.
  NewPing(6, 2, MAX_DISTANCE),
  NewPing(6, 4, MAX_DISTANCE),
  NewPing(7, 8, MAX_DISTANCE),
  NewPing(7, 10, MAX_DISTANCE)
};

boolean proximity;
unsigned int smallestDistance;
unsigned int delayDuration;
unsigned long initialTime;
unsigned long alarmTime;
boolean alarmOn;

void setup() {  
  Serial.begin(115200);
  alarmOn = true;
  initialTime = millis();
}

void loop() {
  proximity = false;
  smallestDistance = 1000;

  for(unsigned i = 0 ; i < SONAR_NUM ; i++) {
    if(proximityAlert(sonar[i], i)) {
      proximity = true;
    }
  }
  
  setAlarm(proximity);
}

boolean proximityAlert(NewPing sensor, int num){
  int distance = sensor.ping_cm();
    
  Serial.print("Sensor ");
  Serial.print(num + 1);
  Serial.print(" : ");
  Serial.println(distance);

  if( distance >= MIN_ALARM_DISTANCE && distance <= MAX_ALARM_DISTANCE ){
    if( distance < smallestDistance ){
      smallestDistance = distance;
    }
    return true;
  }
  return false;
}

void setAlarm(boolean turnOn){
  if( turnOn ){
    unsigned duration = calcDuration( smallestDistance );
    alarmTime = millis() - initialTime;
  
    if( alarmTime > duration ){
      initialTime = millis();
      switchAlarm();
    }
  }
  else{
    noNewTone( SPEAKER_1 );
    alarmOn = false;
  }
}

void switchAlarm(){
  if( alarmOn ){
    noNewTone( SPEAKER_1 );
    alarmOn = false;
  }
  else{
    NewTone( SPEAKER_1, SPEAKER_FREQUENCY );
    alarmOn = true;
  }
}

unsigned int calcDuration(unsigned distance){
  return (slope * distance) + b;
}



